@@ -14,221 +14,35 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+from __future__ import annotations
 
 from functools import partial
-from robocorp_ls_core.robotframework_log import get_logger
-from robotframework_debug_adapter.constants import DEBUG
-from robotframework_ls.options import DEFAULT_TIMEOUT
 import itertools
-import json
 import os.path
 import threading
-import queue
+from typing import Optional
+import typing
+
+from robocorp_ls_core.debug_adapter_core.dap.dap_base_schema import BaseSchema
+from robocorp_ls_core.debug_adapter_core.dap.dap_schema import (
+    LaunchRequest,
+    LaunchResponse,
+    DisconnectRequest,
+)
+from robocorp_ls_core.robotframework_log import get_logger
+from robotframework_debug_adapter.base_launch_process_target import (
+    IProtocolMessageCallable,
+)
+from robotframework_debug_adapter.constants import DEBUG
+from robotframework_ls.options import DEFAULT_TIMEOUT
+
+
+if typing.TYPE_CHECKING:
+    from robotframework_debug_adapter.debug_adapter_comm import DebugAdapterComm
 
 log = get_logger(__name__)
 
 
-class _DebugAdapterRobotTargetComm(threading.Thread):
-    """
-    This class is used so intermediate talking to the server.
-    
-    It's the middle ground between the `DebugAdapterComm` and `RobotTargetComm`.
-        - `DebugAdapterComm`:
-            It's used to talk with the client (in this process) and accessed
-            through the _weak_debug_adapter_comm attribute.
-             
-        - `RobotTargetComm`
-            It's actually in the target process. We communicate with it by 
-            calling the `write_to_robot_message` method and receive messages
-            from it in the `_from_robot` method in this class.
-    """
-
-    def __init__(self, debug_adapter_comm):
-        threading.Thread.__init__(self)
-        import weakref
-
-        self._server_socket = None
-        self._connected_event = threading.Event()
-
-        self._process_event_msg = None
-        self._process_event = threading.Event()
-
-        self._terminated_event_msg = None
-        self._terminated_event = threading.Event()
-
-        self._write_to_robot_queue = queue.Queue()
-        self._weak_debug_adapter_comm = weakref.ref(debug_adapter_comm)
-
-        self._next_seq = partial(next, itertools.count(0))
-        self._msg_id_to_on_response = {}
-
-    def start_listening(self):
-        import socket
-
-        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-        s.bind(("127.0.0.1", 0))
-        port = s.getsockname()[1]
-        s.listen(1)
-        self._server_socket = s
-        self.start()
-        return port
-
-    def run(self):
-        from robocorp_ls_core.debug_adapter_core.debug_adapter_threads import (
-            writer_thread_no_auto_seq,
-        )
-        from robocorp_ls_core.debug_adapter_core.debug_adapter_threads import reader_thread
-
-        try:
-            assert (
-                self._server_socket is not None
-            ), "start_listening must be called before start()"
-
-            # while True:
-            # Only handle a single connection...
-            socket, addr = self._server_socket.accept()
-
-            read_from = socket.makefile("rb")
-            write_to = socket.makefile("wb")
-
-            debug_adapter_comm = self._weak_debug_adapter_comm()
-            writer = self._writer_thread = threading.Thread(
-                target=writer_thread_no_auto_seq,
-                args=(write_to, self._write_to_robot_queue, "write to robot process"),
-                name="Write to robot (_DebugAdapterRobotTargetComm)",
-            )
-            writer.daemon = True
-
-            reader = self._reader_thread = threading.Thread(
-                target=reader_thread,
-                args=(
-                    read_from,
-                    self._from_robot,
-                    debug_adapter_comm.write_to_client_queue,  # Used for errors
-                    b"read from robot process",
-                ),
-                name="Read from robot (_DebugAdapterRobotTargetComm)",
-            )
-            reader.daemon = True
-
-            reader.start()
-            writer.start()
-
-            self._connected_event.set()
-        except:
-            log.exception()
-
-    def _from_robot(self, protocol_message):
-        from robocorp_ls_core.debug_adapter_core.debug_adapter_threads import (
-            READER_THREAD_STOPPED,
-        )
-
-        if protocol_message is READER_THREAD_STOPPED:
-            if DEBUG:
-                log.debug(
-                    "%s when reading from robot: READER_THREAD_STOPPED."
-                    % (self.__class__.__name__,)
-                )
-            return
-
-        if DEBUG:
-            log.debug(
-                "Process json: %s\n"
-                % (json.dumps(protocol_message.to_dict(), indent=4, sort_keys=True),)
-            )
-
-        try:
-            on_response = None
-            if protocol_message.type == "request":
-                method_name = "on_%s_request" % (protocol_message.command,)
-            elif protocol_message.type == "event":
-                method_name = "on_%s_event" % (protocol_message.event,)
-            elif protocol_message.type == "response":
-                on_response = self._msg_id_to_on_response.pop(
-                    protocol_message.request_seq, None
-                )
-                method_name = "on_%s_response" % (protocol_message.command,)
-            else:
-                if DEBUG:
-                    log.debug(
-                        "Unable to decide how to deal with protocol type: %s (read from robot - %s).\n"
-                        % (protocol_message.type, self.__class__.__name__)
-                    )
-                return
-
-            if on_response is not None:
-                on_response(protocol_message)
-
-            on_request = getattr(self, method_name, None)
-
-            if on_request is not None:
-                on_request(protocol_message)
-            elif on_response is not None:
-                pass
-            else:
-                if DEBUG:
-                    log.debug(
-                        "Unhandled: %s not available when reading from robot - %s.\n"
-                        % (method_name, self.__class__.__name__)
-                    )
-        except:
-            log.exception("Error")
-
-    def on_process_event(self, event):
-        self._process_event_msg = event
-        self._process_event.set()
-
-    def get_pid(self):
-        assert self._process_event.is_set()
-        return self._process_event_msg.body.systemProcessId
-
-    def on_stopped_event(self, event):
-        debug_adapter_comm = self._weak_debug_adapter_comm()
-        if debug_adapter_comm is not None:
-            debug_adapter_comm.write_to_client_message(event)
-        else:
-            log.debug("Command processor collected in event: %s" % (event,))
-
-    def on_terminated_event(self, event):
-        from robocorp_ls_core.debug_adapter_core.dap.dap_schema import TerminatedEvent
-        from robocorp_ls_core.debug_adapter_core.dap.dap_schema import TerminatedEventBody
-
-        self._terminated_event_msg = event
-        self._terminated_event.set()
-        self.write_to_robot_message(TerminatedEvent(TerminatedEventBody()))
-
-    def write_to_robot_message(self, protocol_message, on_response=None):
-        """
-        :param BaseSchema protocol_message:
-            Some instance of one of the messages in the debug_adapter.schema.
-        """
-        seq = protocol_message.seq = self._next_seq()
-        if on_response is not None:
-            self._msg_id_to_on_response[seq] = on_response
-        self._write_to_robot_queue.put(protocol_message)
-
-    def wait_for_connection(self):
-        """
-        :return bool:
-            Returns True if the connection was successful and False otherwise.
-        """
-        assert self._server_socket is not None, "start_listening must be called first."
-        log.debug("Wating for connection for %s seconds." % (DEFAULT_TIMEOUT,))
-        ret = self._connected_event.wait(DEFAULT_TIMEOUT)
-        log.debug("Connected: %s" % (ret,))
-        return ret
-
-    def wait_for_process_event(self):
-        log.debug("Wating for process event for %s seconds." % (DEFAULT_TIMEOUT,))
-        ret = self._process_event.wait(DEFAULT_TIMEOUT)
-        log.debug("Received process event: %s" % (ret,))
-        return ret
-
-    def wait_for_process_terminated(self, timeout):
-        return self._terminated_event.wait(timeout)
-
-
 def _noop(*args, **kwargs):
     pass
 
@@ -257,7 +71,7 @@ def _notify_on_exited_pid(on_exit, pid):
             if not is_process_alive(pid):
                 break
 
-            time.sleep(0.5)
+            time.sleep(0.2)
         log.debug("pid exited (_notify_on_exited_pid).")
         on_exit()
     except:
@@ -276,23 +90,38 @@ class LaunchProcess(object):
         "_cwd",
         "_run_in_debug_mode",
         "_debug_adapter_robot_target_comm",
+        "_debug_adapter_pydevd_target_comm",
         "_launch_response",
         "_next_seq",
         "_track_process_pid",
-        "_sent_terminated",
         "_env",
     ]
 
-    def __init__(self, request, launch_response, debug_adapter_comm):
-        """
-        :param LaunchRequest request:
-        :param LaunchResponse launch_response:
-        """
+    def __init__(
+        self,
+        request: LaunchRequest,
+        launch_response: LaunchResponse,
+        debug_adapter_comm: DebugAdapterComm,
+    ) -> None:
+        self._init(request, launch_response, debug_adapter_comm)
+
+    def _init(
+        self,
+        request: LaunchRequest,
+        launch_response: LaunchResponse,
+        debug_adapter_comm: DebugAdapterComm,
+    ) -> None:
         import weakref
         from robotframework_debug_adapter.constants import VALID_TERMINAL_OPTIONS
         from robotframework_debug_adapter.constants import TERMINAL_NONE
         from robocorp_ls_core.basic import as_str
         import robocorp_ls_core
+        from robotframework_debug_adapter.launch_process_robot_target_comm import (
+            LaunchProcessDebugAdapterRobotTargetComm,
+        )
+        from robotframework_debug_adapter.launch_process_pydevd_comm import (
+            LaunchProcessDebugAdapterPydevdComm,
+        )
 
         self._weak_debug_adapter_comm = weakref.ref(debug_adapter_comm)
         self._valid = True
@@ -301,11 +130,6 @@ class LaunchProcess(object):
         self._launch_response = launch_response
         self._next_seq = partial(next, itertools.count(0))
         self._track_process_pid = None
-        self._sent_terminated = threading.Event()
-
-        self._debug_adapter_robot_target_comm = _DebugAdapterRobotTargetComm(
-            debug_adapter_comm
-        )
 
         def mark_invalid(message):
             launch_response.success = False
@@ -373,7 +197,17 @@ class LaunchProcess(object):
         if DEBUG:
             log.debug("Run in debug mode: %s\n" % (self._run_in_debug_mode,))
 
-        port = self._debug_adapter_robot_target_comm.start_listening()
+        self._debug_adapter_robot_target_comm = LaunchProcessDebugAdapterRobotTargetComm(
+            debug_adapter_comm
+        )
+
+        port, server_socket = self._debug_adapter_robot_target_comm.start_listening(
+            2 if self._run_in_debug_mode else 1
+        )
+
+        self._debug_adapter_pydevd_target_comm = LaunchProcessDebugAdapterPydevdComm(
+            debug_adapter_comm, server_socket
+        )
 
         try:
             run_robot_py = os.path.join(
@@ -403,48 +237,67 @@ class LaunchProcess(object):
         self._cmdline = cmdline
 
     @property
-    def valid(self):
+    def valid(self) -> bool:
         return self._valid
 
     @property
-    def run_in_debug_mode(self):
+    def run_in_debug_mode(self) -> bool:
         return self._run_in_debug_mode
 
-    def notify_exit(self):
-        if self._sent_terminated.is_set():
-            return
-        self._sent_terminated.set()
-        from robocorp_ls_core.debug_adapter_core.dap.dap_schema import TerminatedEvent
-        from robocorp_ls_core.debug_adapter_core.dap.dap_schema import TerminatedEventBody
-
-        debug_adapter_comm = self._weak_debug_adapter_comm()
-        if debug_adapter_comm is not None:
-            restart = False
-            terminated_event = TerminatedEvent(
-                body=TerminatedEventBody(restart=restart)
-            )
-            debug_adapter_comm.write_to_client_message(terminated_event)
-
-    def send_and_wait_for_configuration_done_request(self):
-        from robocorp_ls_core.debug_adapter_core.dap.dap_schema import ConfigurationDoneRequest
+    def send_and_wait_for_configuration_done_request(self) -> bool:
+        """
+        :return: Whether the configuration done response was received.
+        """
+        from robocorp_ls_core.debug_adapter_core.dap.dap_schema import (
+            ConfigurationDoneRequest,
+        )
         from robocorp_ls_core.debug_adapter_core.dap.dap_schema import (
             ConfigurationDoneArguments,
         )
 
-        event = threading.Event()
+        event_robot = threading.Event()
+        track_events = [event_robot]
+
         self._debug_adapter_robot_target_comm.write_to_robot_message(
             ConfigurationDoneRequest(ConfigurationDoneArguments()),
-            on_response=lambda *args, **kwargs: event.set(),
+            on_response=lambda *args, **kwargs: event_robot.set(),
         )
+
+        if self._run_in_debug_mode:
+            event_pydevd = threading.Event()
+            track_events.append(event_pydevd)
+            self._debug_adapter_pydevd_target_comm.write_to_pydevd_message(
+                ConfigurationDoneRequest(ConfigurationDoneArguments()),
+                on_response=lambda *args, **kwargs: event_pydevd.set(),
+            )
+
         log.debug(
             "Wating for configuration_done response for %s seconds."
             % (DEFAULT_TIMEOUT,)
         )
-        ret = event.wait(DEFAULT_TIMEOUT)
+        ret = True
+        for event in track_events:
+            ret = ret and event.wait(DEFAULT_TIMEOUT)
+            if not ret:
+                break
         log.debug("Received configuration_done response: %s" % (ret,))
         return ret
 
-    def resend_request_to_robot(self, request):
+    def resend_request_to_stopped_target(self, request: BaseSchema, target) -> None:
+        if target is None:
+            log.debug(f"Detected no paused backend to resend request: {request}")
+            return
+
+        if target is self._debug_adapter_pydevd_target_comm:
+            self.resend_request_to_pydevd(request)
+        elif target is self._debug_adapter_robot_target_comm:
+            self.resend_request_to_robot(request)
+        else:
+            log.debug(
+                f"Stopped target unexpected: {target} for resending request: {request}"
+            )
+
+    def resend_request_to_robot(self, request: BaseSchema) -> None:
         request_seq = request.seq
 
         def on_response(response_msg):
@@ -461,11 +314,45 @@ class LaunchProcess(object):
             request, on_response
         )
 
+    def resend_request_to_pydevd(self, request: BaseSchema) -> None:
+        request_seq = request.seq
+
+        def on_response(response_msg):
+            response_msg.request_seq = request_seq
+            debug_adapter_comm = self._weak_debug_adapter_comm()
+            if debug_adapter_comm is not None:
+                debug_adapter_comm.write_to_client_message(response_msg)
+            else:
+                log.debug(
+                    "Command processor collected in resend request: %s" % (request,)
+                )
+
+        self._debug_adapter_pydevd_target_comm.write_to_pydevd_message(
+            request, on_response
+        )
+
+    def write_to_robot_and_pydevd(self, request: BaseSchema):
+        self._debug_adapter_robot_target_comm.write_to_robot_message(request)
+        if self._run_in_debug_mode:
+            self._debug_adapter_pydevd_target_comm.write_to_pydevd_message(request)
+
+    def write_to_pydevd(
+        self,
+        request: BaseSchema,
+        on_response: Optional[IProtocolMessageCallable] = None,
+    ):
+        if self._run_in_debug_mode:
+            self._debug_adapter_pydevd_target_comm.write_to_pydevd_message(
+                request, on_response=on_response
+            )
+
     def launch(self):
         from robotframework_debug_adapter.constants import TERMINAL_NONE
         from robotframework_debug_adapter.constants import TERMINAL_EXTERNAL
         from robotframework_debug_adapter.constants import TERMINAL_INTEGRATED
-        from robocorp_ls_core.debug_adapter_core.dap.dap_schema import RunInTerminalRequest
+        from robocorp_ls_core.debug_adapter_core.dap.dap_schema import (
+            RunInTerminalRequest,
+        )
         from robocorp_ls_core.debug_adapter_core.dap.dap_schema import (
             RunInTerminalRequestArguments,
         )
@@ -550,14 +437,28 @@ class LaunchProcess(object):
         if not self._debug_adapter_robot_target_comm.wait_for_connection():
             launch_response = self._launch_response
             launch_response.success = False
-            launch_response.message = "Debug adapter timed out waiting for connection."
+            launch_response.message = (
+                "Debug adapter timed out waiting for Robot connection."
+            )
             self._valid = False
 
-        self._debug_adapter_robot_target_comm.write_to_robot_message(
-            InitializeRequest(
-                InitializeRequestArguments("robot-launch-process-adapter")
-            )
+        if self._run_in_debug_mode:
+            # Ok, at this point we connected to the robot target debug adapter,
+            # now, let's connect to pydevd too (if in debug mode).
+            self._debug_adapter_pydevd_target_comm.start()
+            if not self._debug_adapter_pydevd_target_comm.wait_for_connection():
+                launch_response = self._launch_response
+                launch_response.success = False
+                launch_response.message = (
+                    "Debug adapter timed out waiting for pydevd connection."
+                )
+                self._valid = False
+
+        initialize_request = InitializeRequest(
+            InitializeRequestArguments("robot-launch-process-adapter")
         )
+        self.write_to_robot_and_pydevd(initialize_request)
+
         # Note: only start listening stdout/stderr when connected.
         for t in threads:
             t.start()
@@ -572,19 +473,22 @@ class LaunchProcess(object):
         else:
             self._track_process_pid = self._debug_adapter_robot_target_comm.get_pid()
 
-    def after_launch_response_sent(self):
+    def after_launch_response_sent(self) -> None:
         if self._track_process_pid is None:
             log.debug("Unable to track if pid is alive (pid unavailable).")
             return
         t = threading.Thread(
             target=_notify_on_exited_pid,
-            args=(self.notify_exit, self._track_process_pid),
+            args=(
+                self._debug_adapter_robot_target_comm.notify_exit,
+                self._track_process_pid,
+            ),
             name="Track PID alive",
         )
         t.daemon = True
         t.start()
 
-    def disconnect(self, disconnect_request):
+    def disconnect(self, disconnect_request: DisconnectRequest) -> None:
         from robocorp_ls_core.basic import kill_process_and_subprocesses
 
         if self._popen is not None:
