@@ -4,6 +4,7 @@ import socket as socket_module
 import sys
 import threading
 import traceback
+import weakref
 
 try:
     import queue
@@ -72,6 +73,7 @@ class _RobotTargetComm(threading.Thread):
         self._write_queue = queue.Queue()
         self.configuration_done = threading.Event()
         self.terminated = threading.Event()
+        self._run_in_debug_mode = debug
 
         log = get_log()
         if debug:
@@ -102,16 +104,17 @@ class _RobotTargetComm(threading.Thread):
     def _notify_stopped(self):
         from robocorp_ls_core.debug_adapter_core.dap.dap_schema import StoppedEvent
         from robocorp_ls_core.debug_adapter_core.dap.dap_schema import StoppedEventBody
-        from robotframework_debug_adapter.constants import MAIN_THREAD_ID
+
+        thread_id = self.get_current_thread_id()
 
         reason = self._debugger_impl.stop_reason
         body = StoppedEventBody(
-            reason.value, allThreadsStopped=True, threadId=MAIN_THREAD_ID
+            reason.value, allThreadsStopped=True, threadId=thread_id
         )
         msg = StoppedEvent(body)
         self.write_message(msg)
 
-    def start_communication_threads(self):
+    def start_communication_threads(self, mark_as_pydevd_threads):
         from robocorp_ls_core.debug_adapter_core.debug_adapter_threads import (
             writer_thread,
         )
@@ -124,18 +127,30 @@ class _RobotTargetComm(threading.Thread):
 
         writer = self._writer_thread = threading.Thread(
             target=writer_thread,
-            args=(write_to, self._write_queue, "write to dap"),
+            args=(write_to, self._write_queue, "write to dap", True),
             name="Write from robot to dap (_RobotTargetComm)",
         )
         writer.setDaemon(True)
 
         reader = self._reader_thread = threading.Thread(
             target=reader_thread,
-            args=(read_from, self.process_message, self._write_queue, b"read from dap"),
+            args=(
+                read_from,
+                self.process_message,
+                self._write_queue,
+                b"read from dap",
+                True,
+            ),
             name="Read from dap to robot (_RobotTargetComm)",
         )
         reader.setDaemon(True)
 
+        if mark_as_pydevd_threads:
+            import pydevd
+
+            pydevd.mark_as_pydevd_daemon_thread(reader)
+            pydevd.mark_as_pydevd_daemon_thread(writer)
+
         reader.start()
         writer.start()
         return reader, writer
@@ -226,7 +241,6 @@ class _RobotTargetComm(threading.Thread):
         from robocorp_ls_core.debug_adapter_core.dap.dap_base_schema import (
             build_response,
         )
-        from robocorp_ls_core.debug_adapter_core.dap.dap_schema import AttachResponse
 
         attach_response = build_response(request)
         self.write_message(attach_response)
@@ -345,17 +359,27 @@ class _RobotTargetComm(threading.Thread):
         from robocorp_ls_core.debug_adapter_core.dap.dap_schema import (
             ThreadsResponseBody,
         )
-        from robotframework_debug_adapter.constants import MAIN_THREAD_ID
         from robocorp_ls_core.debug_adapter_core.dap.dap_base_schema import (
             build_response,
         )
 
-        threads = [Thread(MAIN_THREAD_ID, "Main Thread").to_dict()]
+        thread_id = self.get_current_thread_id()
+
+        threads = [Thread(thread_id, "Main Thread").to_dict()]
         kwargs = {"body": ThreadsResponseBody(threads)}
         # : :type threads_response: ThreadsResponse
         threads_response = build_response(request, kwargs)
         self.write_message(threads_response)
 
+    def get_current_thread_id(self):
+        if self._run_in_debug_mode:
+            from robotframework_debug_adapter.vendored import force_pydevd  # noqa
+            from _pydevd_bundle.pydevd_constants import get_current_thread_id
+
+            return get_current_thread_id(threading.current_thread())
+        else:
+            return 1
+
     def on_stackTrace_request(self, request):
         """
         :param StackTraceRequest request:
@@ -494,6 +518,7 @@ def main():
     from robocorp_ls_core.robotframework_log import (
         configure_logger,
         log_args_and_python,
+        close_logging_streams,
     )
 
     from robotframework_debug_adapter.constants import LOG_FILENAME
@@ -507,7 +532,7 @@ def main():
 
     args = sys.argv[1:]
     assert args[0] == "--port"
-    port = args[1]
+    port = int(args[1])
     debug = True if args[2] == "--debug" else False
 
     robot_args = args[3:]
@@ -517,9 +542,29 @@ def main():
             "--listener=robotframework_debug_adapter.listeners.DebugListenerV2",
         ] + robot_args
 
-    s = connect(int(port))
+    s = connect(port)
+
+    if debug:
+        # Make sure that we can use pydevd (initialize only in debug mode).
+        import robotframework_debug_adapter.vendored.force_pydevd  # @UnusedImport
+
+        import pydevd
+
+        pydevd.settrace(
+            "127.0.0.1",
+            port=port,
+            suspend=False,
+            trace_only_current_thread=False,
+            overwrite_prev_trace=True,
+            patch_multiprocessing=False,
+            block_until_connected=True,
+            wait_for_ready_to_run=False,
+            notify_stdin=False,
+        )
+
     processor = _RobotTargetComm(s, debug=debug)
-    processor.start_communication_threads()
+    reader, writer = processor.start_communication_threads(debug)
+
     if not processor.configuration_done.wait(DEFAULT_TIMEOUT):
         sys.stderr.write(
             "Process not configured for launch in the available timeout.\n"
@@ -541,6 +586,8 @@ def main():
         processor.terminate()
         if processor.terminated.wait(2):
             log.debug("Processed dap terminate event in robot.")
+        close_logging_streams()
+
     sys.exit(exitcode)
 
 
