@@ -378,6 +378,19 @@ def iter_variables(ast) -> Iterator[NodeInfo]:
         yield NodeInfo(tuple(stack), node)
 
 
+def iter_tests(ast) -> Iterator[NodeInfo]:
+    for stack, node in _iter_nodes_filtered(ast, accept_class="TestCase"):
+        yield NodeInfo(tuple(stack), node)
+
+
+def iter_test_case_sections(ast) -> Iterator[NodeInfo]:
+    # Sections are top-level, so, we don't need to do it recursively.
+    for stack, node in _iter_nodes_filtered(
+        ast, accept_class="TestCaseSection", recursive=False
+    ):
+        yield NodeInfo(tuple(stack), node)
+
+
 def iter_keyword_arguments_as_str(ast) -> Iterator[str]:
     for token in iter_keyword_arguments_as_tokens(ast):
         yield str(token)
@@ -578,3 +591,15 @@ def _append_eol_to_prev_token(last_token, eol_token_contents):
         col_offset=last_token.col_offset,
         error=last_token.error,
     )
+
+
+def create_range_from_token(token):
+    from robocorp_ls_core.lsp import RangeTypedDict, PositionTypedDict
+
+    start: PositionTypedDict = {"line": token.lineno - 1, "character": token.col_offset}
+    end: PositionTypedDict = {
+        "line": token.lineno - 1,
+        "character": token.end_col_offset,
+    }
+    code_lens_range: RangeTypedDict = {"start": start, "end": end}
+    return code_lens_range
