@@ -55,7 +55,7 @@ def complete(completion_context):
                     # text_edit = None
                     items.append(
                         CompletionItem(
-                            label, kind=CompletionItemKind.Class, text_edit=text_edit
+                            label, kind=CompletionItemKind.Class, text_edit=text_edit, insertText=text_edit.newText
                         )
                     )
 
