@@ -180,6 +180,18 @@ class RobotFrameworkLanguageServer(PythonLanguageServer):
         workspaceFolders=None,
         **_kwargs,
     ) -> dict:
+        # capabilities = _kwargs.get("capabilities", {})
+        # text_document_capabilities = capabilities.get("textDocument", {})
+        # document_symbol_capabilities = text_document_capabilities.get(
+        #     "documentSymbol", {}
+        # )
+        # hierarchical_document_symbol_support = document_symbol_capabilities.get(
+        #     "hierarchicalDocumentSymbolSupport", False
+        # )
+        # self._hierarchical_document_symbol_support = (
+        #     hierarchical_document_symbol_support
+        # )
+
         ret = PythonLanguageServer.m_initialize(
             self,
             processId=processId,
@@ -208,22 +220,24 @@ class RobotFrameworkLanguageServer(PythonLanguageServer):
 
         server_capabilities = {
             "codeActionProvider": False,
-            # "codeLensProvider": {
-            #     "resolveProvider": False,  # We may need to make this configurable
-            # },
+            "codeLensProvider": {
+                "resolveProvider": False  # We may need to make this configurable
+            },
             "completionProvider": {
                 "resolveProvider": False  # We know everything ahead of time
             },
             "documentFormattingProvider": True,
             "documentHighlightProvider": False,
             "documentRangeFormattingProvider": False,
-            "documentSymbolProvider": False,
+            "documentSymbolProvider": True,
             "definitionProvider": True,
             "executeCommandProvider": {
                 "commands": [
                     "robot.addPluginsDir",
                     "robot.resolveInterpreter",
                     "robot.getLanguageServerVersion",
+                    "robot.getInternalInfo",
+                    "robot.listTests",
                 ]
             },
             "hoverProvider": True,
@@ -302,6 +316,23 @@ class RobotFrameworkLanguageServer(PythonLanguageServer):
         elif command == "robot.getLanguageServerVersion":
             return __version__
 
+        elif command == "robot.listTests":
+            doc_uri = arguments[0]["uri"]
+
+            rf_api_client = self._server_manager.get_others_api_client(doc_uri)
+            if rf_api_client is not None:
+                func = partial(
+                    self._async_api_request,
+                    rf_api_client,
+                    "request_list_tests",
+                    doc_uri=doc_uri,
+                )
+                func = require_monitor(func)
+                return func
+
+            log.info("Unable to list tests (no api available).")
+            return []
+
     @overrides(PythonLanguageServer.m_workspace__did_change_configuration)
     @log_and_silence_errors(log)
     def m_workspace__did_change_configuration(self, **kwargs):
@@ -642,6 +673,40 @@ class RobotFrameworkLanguageServer(PythonLanguageServer):
         log.info("Unable to get folding range (no api available).")
         return []
 
+    def m_text_document__code_lens(self, **kwargs):
+        doc_uri = kwargs["textDocument"]["uri"]
+
+        rf_api_client = self._server_manager.get_others_api_client(doc_uri)
+        if rf_api_client is not None:
+            func = partial(
+                self._async_api_request,
+                rf_api_client,
+                "request_code_lens",
+                doc_uri=doc_uri,
+            )
+            func = require_monitor(func)
+            return func
+
+        log.info("Unable to get code lens (no api available).")
+        return []
+
+    def m_text_document__document_symbol(self, **kwargs):
+        doc_uri = kwargs["textDocument"]["uri"]
+
+        rf_api_client = self._server_manager.get_others_api_client(doc_uri)
+        if rf_api_client is not None:
+            func = partial(
+                self._async_api_request,
+                rf_api_client,
+                "request_document_symbol",
+                doc_uri=doc_uri,
+            )
+            func = require_monitor(func)
+            return func
+
+        log.info("Unable to get document symbol (no api available).")
+        return []
+
     def m_text_document__hover(self, **kwargs):
         doc_uri = kwargs["textDocument"]["uri"]
         # Note: 0-based
