@@ -2,7 +2,7 @@ from robocorp_ls_core.python_ls import PythonLanguageServer
 from robocorp_ls_core.basic import overrides
 from robocorp_ls_core.robotframework_log import get_logger
 from typing import Optional, List, Dict
-from robocorp_ls_core.protocols import IConfig, IMonitor
+from robocorp_ls_core.protocols import IConfig, IMonitor, ITestInfoTypedDict
 from functools import partial
 from robocorp_ls_core.jsonrpc.endpoint import require_monitor
 from robocorp_ls_core.lsp import (
@@ -10,6 +10,8 @@ from robocorp_ls_core.lsp import (
     FoldingRangeTypedDict,
     HoverTypedDict,
     TextDocumentTypedDict,
+    CodeLensTypedDict,
+    DocumentSymbolTypedDict,
 )
 from robotframework_ls.impl.protocols import IKeywordFound
 
@@ -382,6 +384,54 @@ class RobotFrameworkServerApi(PythonLanguageServer):
 
         return folding_range(completion_context)
 
+    def m_code_lens(self, doc_uri: str):
+        func = partial(self._threaded_code_lens, doc_uri)
+        func = require_monitor(func)
+        return func
+
+    def _threaded_code_lens(
+        self, doc_uri: str, monitor: IMonitor
+    ) -> List[CodeLensTypedDict]:
+        from robotframework_ls.impl.code_lens import code_lens
+
+        completion_context = self._create_completion_context(doc_uri, 0, 0, monitor)
+        if completion_context is None:
+            return []
+
+        return code_lens(completion_context)
+
+    def m_document_symbol(self, doc_uri: str):
+        func = partial(self._threaded_document_symbol, doc_uri)
+        func = require_monitor(func)
+        return func
+
+    def _threaded_document_symbol(
+        self, doc_uri: str, monitor: IMonitor
+    ) -> List[DocumentSymbolTypedDict]:
+        from robotframework_ls.impl.document_symbol import document_symbol
+
+        completion_context = self._create_completion_context(doc_uri, 0, 0, monitor)
+        if completion_context is None:
+            return []
+
+        return document_symbol(completion_context)
+
+    def m_list_tests(self, doc_uri: str):
+        func = partial(self._threaded_list_tests, doc_uri)
+        func = require_monitor(func)
+        return func
+
+    def _threaded_list_tests(
+        self, doc_uri: str, monitor: IMonitor
+    ) -> List[ITestInfoTypedDict]:
+        from robotframework_ls.impl.code_lens import list_tests
+
+        completion_context = self._create_completion_context(doc_uri, 0, 0, monitor)
+        if completion_context is None:
+            return []
+
+        return list_tests(completion_context)
+
     def m_hover(self, doc_uri: str, line: int, col: int):
         func = partial(self._threaded_hover, doc_uri, line, col)
         func = require_monitor(func)
