@@ -310,6 +310,12 @@ class IRobotFrameworkApiClient(ILanguageServerClientBase, Protocol):
         pass
 
 
+class ITestInfoTypedDict(TypedDict):
+    uri: str
+    path: str
+    name: str
+
+
 class ILanguageServerClient(ILanguageServerClientBase, Protocol):
     pid: Optional[int]
 
@@ -384,6 +390,12 @@ class ILanguageServerClient(ILanguageServerClientBase, Protocol):
     def request_folding_range(self, uri: str):
         pass
 
+    def request_code_lens(self, uri: str):
+        pass
+
+    def request_document_symbol(self, uri: str):
+        pass
+
     def request_workspace_symbols(self, query: Optional[str] = None):
         pass
 
